/*============================================================================*/
/*                                                                            */
/*                                                                            */
/*                              reLOC 0.08-osaka                              */
/*                                                                            */
/*                   (C) Copyright 2011-2013 Pavel Surynek                    */
/*            http://www.surynek.com | <pavel.surynek@mff.cuni.cz>            */
/*                                                                            */
/*                                                                            */
/*============================================================================*/
// gridgen_main.cpp / 0.08-osaka_038
/*----------------------------------------------------------------------------*/
//
// Grid Instance Generator - main program.
//
// Generated a Multi Robot instance on a 4-connected grid with randomly
// placed obstacles.
//
/*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/times.h>
#include <unistd.h>

#include "config.h"
#include "compile.h"
#include "defs.h"
#include "reloc.h"
#include "multirobot.h"
#include "compress.h"
#include "statistics.h"
#include "version.h"

#include "gridgen_main.h"


using namespace sReloc;


/*----------------------------------------------------------------------------*/

namespace sReloc
{


/*----------------------------------------------------------------------------*/

  sCommandParameters::sCommandParameters()
      : m_x_size(4)
      , m_y_size(4)
      , m_N_robots(5)
      , m_seed(0)
      , m_obstacle_probability(0.1)
      , m_cnf_level(6)
  {
      // nothing
  }


/*----------------------------------------------------------------------------*/

    void print_IntroductoryMessage(void)
    {
	printf("==================================================================\n");
	printf("gridMRPP %s : Multi-Robot Path Planning on Grids as SAT\n", sVERSION);
	printf("%s\n", sCOPYRIGHT);
	printf("------------------------------------------------------------------\n");
    }


    void print_ConcludingMessage(void)
    {
	printf("----------------------------------------------------------------\n");
    }


    void print_Help(void)
    {
	printf("Usage:\n");
	printf("gridMRPP  --x-size=<int>\n");
	printf("          --y-size=<int>\n");
	printf("          --N-robots=<int>\n");
	printf("          --obstacle-probability=<double>\n");
	printf("          --cnf-file=<string>\n");
	printf("          --cnf-level=<int>\n");
	printf("         [--seed=<int>\n");
	printf("         [--pddl-domain-file=<string>]\n");
	printf("         [--pddl-problem-file=<string>]\n");
	printf("\n");
	printf("Examples:\n");
	printf("gridMRPP --x-size=4\n");
	printf("         --y-size=4\n");
	printf("         --N-robots=5\n");
	printf("         --seed=12345\n");
	printf("         --obstacle-probability=0.1\n");
	printf("         --cnf-file=mrpp_4x4#5_6.cnf\n");
	printf("         --cnf-level=6\n");
	printf("\n");
	printf("Defaults: --x-size=4\n");
	printf("          --y-size=4\n");
	printf("          --N-robots=5\n");
	printf("          --seed=0\n");
	printf("          --obstacle-probability=0.1\n");
	printf("          --cnf-level=6\n");
	printf("\n");
    }


    sResult generate_GridInstance(const sCommandParameters &parameters)
    {
	sResult result;

	srand(parameters.m_seed);

	sUndirectedGraph environment(parameters.m_x_size, parameters.m_y_size, parameters.m_obstacle_probability);
	sRobotArrangement initial_arrangement(environment.get_VertexCount(), sMIN(parameters.m_N_robots, environment.get_VertexCount()), true);
	sRobotArrangement goal_arrangement(environment.get_VertexCount(), sMIN(parameters.m_N_robots, environment.get_VertexCount()), true);
	sMultirobotInstance instance(environment, initial_arrangement, goal_arrangement);
	
	if (!parameters.m_cnf_filename.empty())
	{
	    sMultirobotEncodingContext_CNFsat encoding_context(parameters.m_cnf_level);

	    result = instance.to_File_HeuristicAdvancedCNFsat(parameters.m_cnf_filename, encoding_context, "");

	    if (sFAILED(result))
	    {
		printf("Error: Failed to write CNF file %s (code = %d).\n", parameters.m_cnf_filename.c_str(), result);
		return result;
	    }
	}
	if (!parameters.m_pddl_problem_filename.empty())
	{
	    result = instance.to_File_problemPDDL(parameters.m_pddl_problem_filename);

	    if (sFAILED(result))
	    {
		printf("Error: Failed to write PDDL problem file %s (code = %d).\n", parameters.m_pddl_problem_filename.c_str(), result);
		return result;
	    }
	}
	if (!parameters.m_pddl_domain_filename.empty())
	{
	    result = instance.to_File_domainPDDL(parameters.m_pddl_domain_filename);

	    if (sFAILED(result))
	    {
		printf("Error: Failed to write PDDL domain file %s (code = %d).\n", parameters.m_pddl_domain_filename.c_str(), result);
		return result;
	    }
	}
	return sRESULT_SUCCESS;
    }


    sResult parse_CommandLineParameter(const sString &parameter, sCommandParameters &command_parameters)
    {
	if (parameter.find("--x-size=") == 0)
	{
	    command_parameters.m_x_size = sInt_32_from_String(parameter.substr(9, parameter.size()));
	}
	else if (parameter.find("--y-size=") == 0)
	{
	    command_parameters.m_y_size = sInt_32_from_String(parameter.substr(9, parameter.size()));
	}
	else if (parameter.find("--N-robots=") == 0)
	{
	    command_parameters.m_N_robots = sInt_32_from_String(parameter.substr(11, parameter.size()));
	}
	else if (parameter.find("--seed=") == 0)
	{
	    command_parameters.m_seed = sInt_32_from_String(parameter.substr(7, parameter.size()));
	}
	else if (parameter.find("--obstacle-probability=") == 0)
	{
	    command_parameters.m_obstacle_probability = sDouble_from_String(parameter.substr(23, parameter.size()));
	}
	else if (parameter.find("--cnf-level=") == 0)
	{
	    command_parameters.m_cnf_level = sInt_32_from_String(parameter.substr(12, parameter.size()));
	}
	else if (parameter.find("--cnf-file=") == 0)
	{
	    command_parameters.m_cnf_filename = parameter.substr(11, parameter.size());
	}
	else if (parameter.find("--pddl-domain-file=") == 0)
	{
	    command_parameters.m_pddl_domain_filename = parameter.substr(19, parameter.size());
	}
	else if (parameter.find("--pddl-problem-file=") == 0)
	{
	    command_parameters.m_pddl_problem_filename = parameter.substr(20, parameter.size());
	}
	else
	{
	    return sRESOLVER_PROGRAM_UNRECOGNIZED_PARAMETER_ERROR;
	}
	return sRESULT_SUCCESS;
    }


/*----------------------------------------------------------------------------*/

} // namespace sReloc


/*----------------------------------------------------------------------------*/
// main program

int main(int argc, char **argv)
{
    sResult result;
    sCommandParameters command_parameters;

    print_IntroductoryMessage();

    if (argc >= 2 && argc <= 9)
    {
	for (int i = 1; i < argc; ++i)
	{
	    result = parse_CommandLineParameter(argv[i], command_parameters);
	    if (sFAILED(result))
	    {
		printf("Error: Cannot parse command line parameters (code = %d).\n", result);
		print_Help();

		return result;
	    }
	}
	result = generate_GridInstance(command_parameters);
	if (sFAILED(result))
	{
	    return result;
	}
    }
    else
    {
	print_Help();
    }
    printf("Finished.\n");

    return sRESULT_SUCCESS;
}

